! NOTE: modify hunt_s if you use pgf90

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! Collection of frequently used routines of Numerical    !
! Recipe in F90, sometimes slightly modified, and other  !
! numerical routines.                                    !
! 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! INTERPOLATION BENCHMARKs (seconds)
!           liin        poin(mi=2)             poin(mi=2)
!                       using polint77          using polint
!
!ASTERIX   0.32         0.40                    0.48
!PC P133   1.2          3.57                    10.0
!
!NOTE: the increase in cpu time from left to right
!is mild for DEC unix systems, but dramatic for PCs
!therefore 1) don't use poin if only linear interpolation
!is required and 2) in higher order interpolation use polint77
!rather than polint. Thanks. G.L.

MODULE nr90_gian  ! Gian's numerical routines

contains

!**************************************
	SUBROUTINE hunt_s(xx,n,x,jlo)
	USE nrtype
	IMPLICIT NONE
	INTEGER(I4B), INTENT(INOUT) :: jlo
	INTEGER(I4B), INTENT(IN) :: n
	REAL(SP), INTENT(IN) :: x
	REAL(SP), DIMENSION(:), INTENT(IN) :: xx ! better for digital fortran
!	REAL(SP), DIMENSION(N), INTENT(IN) :: xx ! better for PGF90
	INTEGER(I4B) :: inc,jhi,jm
	LOGICAL :: ascnd
	ascnd = (xx(n) >= xx(1))
	if (jlo <= 0 .or. jlo > n) then
		jlo=0
		jhi=n+1
	else
		inc=1
		if (x >= xx(jlo) .eqv. ascnd) then
			do
				jhi=jlo+inc
				if (jhi > n) then
					jhi=n+1
					exit
				else
					if (x < xx(jhi) .eqv. ascnd) exit
					jlo=jhi
					inc=inc+inc
				end if
			end do
		else

			jhi=jlo
			do
				jlo=jhi-inc
				if (jlo < 1) then
					jlo=0
					exit
				else
					if (x >= xx(jlo) .eqv. ascnd) exit
					jhi=jlo
					inc=inc+inc
				end if
			end do
		end if
	end if
	do
		if (jhi-jlo <= 1) then
			if (x == xx(n)) jlo=n-1
			if (x == xx(1)) jlo=1
			exit
		else
			jm=(jhi+jlo)/2
			if (x >= xx(jm) .eqv. ascnd) then
				jlo=jm
			else
				jhi=jm
			end if
		end if
	end do
	END SUBROUTINE hunt_s



!****************************
    function arrint(xa,ya)
!   By Gian Luigi - Versione 23-1-98
!   given arrays XA(M) YA(M) returns 
!    /
!   ! ya dxa
!  /
!
!   computed according to trapezoidal rule
!   on serial machine arrint90 is slower than arrint 
!   (about a factor 2 with a PC and Fortran PSW, 
!   only a few % on alpha), but it is more elegant
!   because the do loop is replaced by an array operation. 
!   May be arrint90 turns out to be better on
!   parallel machines.
    USE nrtype; USE nrutil, ONLY: assert_eq
    IMPLICIT NONE
    REAL(SP) arrint,result
    REAL(SP), DIMENSION(:):: xa,ya

    integer(i4b)::n,i
                                                
	n=assert_eq(size(xa),size(ya),'arrint')

    result=ya(1)*(xa(2)-xa(1))+ya(n)*(xa(n)-xa(n-1))
    do i=2,n-1
        result=result+ya(i)*(xa(i+1)-xa(i-1))
    enddo
    arrint=0.5*result
    

    end function arrint


!****************************
    function arrint90(xa,ya)
!   By Gian Luigi - Versione 23-1-98
!   given arrays XA(M) YA(M) returns 
!    /
!   ! ya dxa
!  /
!
!   computed according to trapezoidal rule
!   on serial machine arrint90 is slower than arrint 
!   (about a factor 2 with a PC and Fortran PSW, 
!   only a few % on alpha), but it is more elegant
!   because the do loop is replaced by an array operation. 
!   May be arrint90 turns out to be better on
!   parallel machines.

    USE nrtype; USE nrutil, ONLY: assert_eq
    IMPLICIT NONE
    REAL(SP) arrint90
    REAL(SP), DIMENSION(:):: xa,ya


    integer(i4b)::n
                                                
	n=assert_eq(size(xa),size(ya),'arrint')


    arrint90=0.5*( sum( ya(2:n-1) * (xa(3:n)-xa(1:n-2)) ) + &
    ya(1)*(xa(2)-xa(1))+ya(n)*(xa(n)-xa(n-1)) )

    end function arrint90



!****************************                              
    SUBROUTINE biliin(x1a,x2a,ya,x1,x2,y,io,jo)
!   By Gian Luigi - Versione 16-12-97
!   dati gli arrays X1A(M) e X1A(N) delle variabili indipendenti,
!   e l'array YA(M,N) della funzione, ritorna Y, ovvero il valore 
!   della funzione interpolato bilinearmente in (X1,X2).
!   Usa la  HUNT di Numerical Recipes.
	USE nrtype; USE nrutil, ONLY: assert_eq
!    USE nr, ONLY : hunt
    IMPLICIT NONE
    REAL(SP), DIMENSION(:), INTENT(IN):: x1a,x2a
    REAL(SP), DIMENSION(:,:), INTENT(IN):: ya
    REAL(SP), INTENT(IN):: x1,x2
    REAL(SP), INTENT(OUT)::y
    INTEGER(I4B), INTENT(INOUT), OPTIONAL::io,jo

    REAL(SP)::t,u
    INTEGER(I4B)::i,j,m,n
    INTEGER(I4B), SAVE::ii,jj
                                                
	m=assert_eq(size(x1a),size(ya,1),'biliin')
	n=assert_eq(size(x2a),size(ya,2),'biliin')

!   localizziamo X1 e X2 nei rispettivi arrays
    if (present(io)) then
	    call hunt_s(x1a,m,x1,io)
        ii=io
    else
	    call hunt_s(x1a,m,x1,ii)
    endif

    if (present(jo)) then
	    call hunt_s(x2a,n,x2,jo)
        jj=jo
    else
	    call hunt_s(x2a,n,x2,jj)
    endif


    if (ii.eq.m) then
        i=m-1
    else if (ii.eq.0) then
        i=1
    else
        i=ii
    endif


    if (jj.eq.n) then
        j=n-1
    else if (jj.eq.0) then
        j=1
    else
        j=jj
    endif

!     finalmente si interpola (num. rec. pag. 96)

    t=(x1-x1a(i))/(x1a(i+1)-x1a(i))
    u=(x2-x2a(j))/(x2a(j+1)-x2a(j))

    y=(1-t)*(1-u)*ya(i,j)+t*(1-u)*ya(i+1,j)+ &
    t*u*ya(i+1,j+1)+(1-t)*u*ya(i,j+1)

    END SUBROUTINE biliin

!****************************
    SUBROUTINE liin(xa,ya,x,y,io)
!   By Gian Luigi - Versione 16-12-97
!   dati gli arrays XA(M) YA(M) della funzione, ritorna Y, ovvero il valore 
!   della funzione interpolato bilinearmente in (X1,X2).
!   Usa la  HUNT di Numerical Recipes.
!
!   NOTA BENE che questa e piu' veloce della generica
!   poin usata con mi=2, anche se da gli stessi risultati:

	USE nrtype; USE nrutil, ONLY: assert_eq
!    USE nr, ONLY :hunt
    IMPLICIT NONE
    REAL(SP), DIMENSION(:), INTENT(IN):: xa,ya
    REAL(SP), INTENT(IN):: x
    REAL(SP), INTENT(OUT)::y
    INTEGER(I4B), INTENT(INOUT), OPTIONAL::io

    INTEGER(I4B)::i,m
    INTEGER(I4B), SAVE::ii
                                                
    m=assert_eq(size(xa),size(ya),'liin')

    if (m>1) then
    !   localizziamo x negl arrays
        if (present(io)) then
	        call hunt_s(xa,m,x,io)
            ii=io
        else
	        call hunt_s(xa,m,x,ii)
        endif

        if (ii.eq.m) then
            i=m-1
        else if (ii.eq.0) then
            i=1
        else
            i=ii
        endif

    !     finalmente si interpola


        y=ya(i)+(ya(i+1)-ya(i))*(x-xa(i))/(xa(i+1)-xa(i))
    else
        y=ya(1)
    endif

    END SUBROUTINE liin



!****************************
	SUBROUTINE poin(x1a,ya,mi,x1,y,dy,io)

!	By Gian Luigi - Versione 17-12-97
!	dati l'array x1a della variabile indipendente,
!	e l'array ya della funzione, ritorna y, ovvero il valore 
!	della funzione interpolato in x1m ed una valutazione
!   dell'errore nel parametro OPZIONALE dy. Anche i e'
!   opzionale, 
!	l'interpolazione e` polinomiale di grado mi-1 
!   (mi al massimo mmax), fatta usando gli mi punti piu` opportuni..
!	usa la hunt di numerical recipes in f90.
!   INVECE USA LA POLINT, MESSA COME SUA PROCEDURA INTERNA, DI
!   NUMERICAL RECIPES IN F77: PER QUALCHE CACCHIO DI RAGIONE
!   CHE NON SO BENE, (comunque e tra' le operazione di array)
!   E 3' VOLTE PIU' VELOCE CON MICROSOFT PWS 
!   SU UNIX LA DIFFERENZA E' MOLTO PIU' MODESTA, ATTORNO AL 20%

	USE nrtype; USE nrutil, ONLY: assert_eq
    USE nr, ONLY : polint,hunt
    IMPLICIT NONE

	REAL(SP),DIMENSION(:), INTENT(IN) :: x1a,ya
    INTEGER(I4B), INTENT(IN)::mi
    REAL(SP), INTENT(IN)::x1
    INTEGER(I4B), INTENT(INOUT), OPTIONAL::io
    REAL, INTENT(OUT):: y
    REAL(SP), INTENT(out), OPTIONAL:: dy
    
	INTEGER(I4B), PARAMETER:: mmax=10
    INTEGER(I4B):: m,mit,mprima,mdopo,ibeg
    INTEGER(I4B), SAVE:: ii
	REAL(SP):: x1ai(mi),yai(mi),dyy


	m=assert_eq(size(x1a),size(ya),'poin')

	mit=min(mi,mmax,m)

!	localizziamo x1 nel suo array,
!	e quindi prepariamo il "sottoarray" da usare nell'interpolazione
    if (present(io)) then
	    call hunt(x1a,x1,io)
        ii=io
    else
	    call hunt(x1a,x1,ii)
    endif

	mprima=mit/2
	mdopo=mit-mprima

	if (ii.lt.mprima) then 
		ibeg=1
	else if (ii.gt.(m-mdopo)) then 
		ibeg=m-mit+1
	else 
		ibeg=ii-mprima+1
	endif

    x1ai=x1a(ibeg:ibeg+mit-1)
    yai=ya(ibeg:ibeg+mit-1)

!	finalmente si interpola

!	call polint(x1ai,yai,x1,y,dyy)
	call polint77(x1ai,yai,mit,x1,y,dyy)

    if (present(dy)) then
        dy=dyy
    endif


	END SUBROUTINE POIN


    !************************
    SUBROUTINE POLINT77(XA,YA,N,X,Y,DY)
	USE nrtype

! this is about a factor three faster than f90 version on a PC
    INTEGER(I4B), PARAMETER:: NMAX=10
    INTEGER(I4B)::N
    REAL(SP), DIMENSION(N):: XA,YA
    REAL(SP), DIMENSION(NMAX):: C,D
    INTEGER(I4B)::I,NS,M
    REAL(SP)::X,Y,DY,DIF,DIFT,DEN,HO,HP,W

       NS=1
       DIF=ABS(X-XA(1))
       DO 11 I=1,N 
         DIFT=ABS(X-XA(I))
         IF (DIFT.LT.DIF) THEN
           NS=I
           DIF=DIFT
         ENDIF
         C(I)=YA(I)
         D(I)=YA(I)
 11    CONTINUE
       Y=YA(NS)
       NS=NS-1
       DO 13 M=1,N-1
         DO 12 I=1,N-M
           HO=XA(I)-X
           HP=XA(I+M)-X
           W=C(I+1)-D(I)
           DEN=HO-HP
           IF(DEN.EQ.0.)PAUSE
           DEN=W/DEN
           D(I)=HP*DEN
           C(I)=HO*DEN
 12      CONTINUE
         IF (2*NS.LT.N-M)THEN
           DY=C(NS+1)
         ELSE
           DY=D(NS)
           NS=NS-1
         ENDIF
         Y=Y+DY
 13    CONTINUE
       RETURN
    END SUBROUTINE POLINT77


    !************************
	FUNCTION qromb_eps(func,a,b,eps)
!   qromb modified to use polint77 instead of polint 
!   (faster on PCs) and to pass accuracy as a parameter
	USE nrtype; USE nrutil, ONLY : nrerror
	USE nr, ONLY : polint,trapzd
	IMPLICIT NONE
	REAL(SP), INTENT(IN) :: a,b,eps
	REAL(SP) :: qromb_eps
	INTERFACE
		FUNCTION func(x)
		USE nrtype
		REAL(SP), DIMENSION(:), INTENT(IN) :: x
		REAL(SP), DIMENSION(size(x)) :: func
		END FUNCTION func
	END INTERFACE
	INTEGER(I4B), PARAMETER :: JMAX=20,JMAXP=JMAX+1,K=5,KM=K-1
	REAL(SP), DIMENSION(JMAXP) :: h,s
	REAL(SP) :: dqromb
	INTEGER(I4B) :: j
	h(1)=1.0
	do j=1,JMAX
		call trapzd(func,a,b,s(j),j)
		if (j >= K) then
!			call polint(h(j-KM:j),s(j-KM:j),0.0_sp,qromb_eps,dqromb)
			call polint77(h(j-KM:j),s(j-KM:j),K,0.0_sp,qromb_eps,dqromb)
			if (abs(dqromb) <= EPS*abs(qromb_eps)) RETURN
		end if
		s(j+1)=s(j)
		h(j+1)=0.25_sp*h(j)
	end do
	write(*,*) 'qromb_eps: too many steps'
	!call nrerror('qromb_eps: too many steps')
	END FUNCTION qromb_eps




    !************************
	FUNCTION qtrap_eps(func,a,b,eps)
!   qtrap modified to pass accuracy as a parameter
	USE nrtype; USE nrutil, ONLY : nrerror
	USE nr, ONLY : trapzd
	IMPLICIT NONE
	REAL(SP), INTENT(IN) :: a,b,eps
	REAL(SP) :: qtrap_eps
	INTERFACE
		FUNCTION func(x)
		USE nrtype
		REAL(SP), DIMENSION(:), INTENT(IN) :: x
		REAL(SP), DIMENSION(size(x)) :: func
		END FUNCTION func
	END INTERFACE
	INTEGER(I4B), PARAMETER :: JMAX=20
	REAL(SP), PARAMETER :: UNLIKELY=-1.0e30_sp
	REAL(SP) :: olds
	INTEGER(I4B) :: j
	olds=UNLIKELY
	do j=1,JMAX
		call trapzd(func,a,b,qtrap_eps,j)
		if (abs(qtrap_eps-olds) < EPS*abs(olds)) RETURN
		if (qtrap_eps == 0.0 .and. olds == 0.0 .and. j > 6) RETURN
		olds=qtrap_eps
	end do
	!call nrerror('qtrap_eps: too many steps')
	write(*,*) 'qtrap_eps: too many steps'

	END FUNCTION qtrap_eps


!***********
	RECURSIVE FUNCTION qgausr(func,a,b)
	USE nrtype
    REAL(SP), INTENT(IN) :: a,b
	REAL(SP) :: qgausr
	INTERFACE
		FUNCTION func(x)
		USE nrtype
		REAL(SP), DIMENSION(:), INTENT(IN) :: x
		REAL(SP), DIMENSION(size(x)) :: func
		END FUNCTION func
	END INTERFACE
	REAL(SP) :: xm,xr
	REAL(SP), DIMENSION(5) :: dx, w = (/ 0.2955242247_sp,0.2692667193_sp,&
		0.2190863625_sp,0.1494513491_sp,0.0666713443_sp /),&
		x = (/ 0.1488743389_sp,0.4333953941_sp,0.6794095682_sp,&
		0.8650633666_sp,0.9739065285_sp /)
	xm=0.5_sp*(b+a)
	xr=0.5_sp*(b-a)
	dx(:)=xr*x(:)
	qgausr=xr*sum(w(:)*(func(xm+dx)+func(xm-dx)))
	END FUNCTION qgausr



END MODULE nr90_gian



!**************************************
	SUBROUTINE hunt(xx,x,jlo)
	USE nrtype
	IMPLICIT NONE
	INTEGER(I4B), INTENT(INOUT) :: jlo
	REAL(SP), INTENT(IN) :: x
	REAL(SP), DIMENSION(:), INTENT(IN) :: xx
	INTEGER(I4B) :: n,inc,jhi,jm
	LOGICAL :: ascnd
	n=size(xx)
	ascnd = (xx(n) >= xx(1))
	if (jlo <= 0 .or. jlo > n) then
		jlo=0
		jhi=n+1
	else
		inc=1
		if (x >= xx(jlo) .eqv. ascnd) then
			do
				jhi=jlo+inc
				if (jhi > n) then
					jhi=n+1
					exit
				else
					if (x < xx(jhi) .eqv. ascnd) exit
					jlo=jhi
					inc=inc+inc
				end if
			end do
		else

			jhi=jlo
			do
				jlo=jhi-inc
				if (jlo < 1) then
					jlo=0
					exit
				else
					if (x >= xx(jlo) .eqv. ascnd) exit
					jhi=jlo
					inc=inc+inc
				end if
			end do
		end if
	end if
	do
		if (jhi-jlo <= 1) then
			if (x == xx(n)) jlo=n-1
			if (x == xx(1)) jlo=1
			exit
		else
			jm=(jhi+jlo)/2
			if (x >= xx(jm) .eqv. ascnd) then
				jlo=jm
			else
				jhi=jm
			end if
		end if
	end do
	END SUBROUTINE hunt



!**************************************
	FUNCTION locate(xx,x)
	USE nrtype
	IMPLICIT NONE
	REAL(SP), DIMENSION(:), INTENT(IN) :: xx
	REAL(SP), INTENT(IN) :: x
	INTEGER(I4B) :: locate
	INTEGER(I4B) :: n,jl,jm,ju
	LOGICAL :: ascnd
	n=size(xx)
	ascnd = (xx(n) >= xx(1))
	jl=0
	ju=n+1
	do
		if (ju-jl <= 1) exit
		jm=(ju+jl)/2
		if (ascnd .eqv. (x >= xx(jm))) then
			jl=jm
		else
			ju=jm
		end if
	end do
	if (x == xx(1)) then
		locate=1
	else if (x == xx(n)) then
		locate=n-1
	else
		locate=jl
	end if
	END FUNCTION locate


!**************************************
MODULE ode_path
	USE nrtype
	INTEGER(I4B) :: nok,nbad,kount
	LOGICAL(LGT), SAVE :: save_steps=.false.
	REAL(SP) :: dxsav
	REAL(SP), DIMENSION(:), POINTER :: xp
	REAL(SP), DIMENSION(:,:), POINTER :: yp
END MODULE ode_path
   
	SUBROUTINE odeint(ystart,x1,x2,eps,h1,hmin,derivs,rkqs)
	USE nrtype; USE nrutil, ONLY : nrerror,reallocate
	USE ode_path
	IMPLICIT NONE
	REAL(SP), DIMENSION(:), INTENT(INOUT) :: ystart
	REAL(SP), INTENT(IN) :: x1,x2,eps,h1,hmin
	INTERFACE
		SUBROUTINE derivs(x,y,dydx)
		USE nrtype
		IMPLICIT NONE
		REAL(SP), INTENT(IN) :: x
		REAL(SP), DIMENSION(:), INTENT(IN) :: y
		REAL(SP), DIMENSION(:), INTENT(OUT) :: dydx
		END SUBROUTINE derivs
!BL
		SUBROUTINE rkqs(y,dydx,x,htry,eps,yscal,hdid,hnext,derivs)
		USE nrtype
		IMPLICIT NONE
		REAL(SP), DIMENSION(:), INTENT(INOUT) :: y
		REAL(SP), DIMENSION(:), INTENT(IN) :: dydx,yscal
		REAL(SP), INTENT(INOUT) :: x
		REAL(SP), INTENT(IN) :: htry,eps
		REAL(SP), INTENT(OUT) :: hdid,hnext
		INTERFACE
			SUBROUTINE derivs(x,y,dydx)
			USE nrtype
			IMPLICIT NONE
			REAL(SP), INTENT(IN) :: x
			REAL(SP), DIMENSION(:), INTENT(IN) :: y
			REAL(SP), DIMENSION(:), INTENT(OUT) :: dydx
			END SUBROUTINE derivs
		END INTERFACE

		END SUBROUTINE rkqs
	END INTERFACE
	REAL(SP), PARAMETER :: TINY=1.0e-30_sp
	INTEGER(I4B), PARAMETER :: MAXSTP=10000
	INTEGER(I4B) :: nstp
	REAL(SP) :: h,hdid,hnext,x,xsav
	REAL(SP), DIMENSION(size(ystart)) :: dydx,y,yscal
	x=x1
	h=sign(h1,x2-x1)
	nok=0
	nbad=0
	kount=0
	y(:)=ystart(:)
	if (save_steps) then
		xsav=x-2.0_sp*dxsav
		nullify(xp,yp)
		allocate(xp(256))
		allocate(yp(size(ystart),size(xp)))
	end if
	do nstp=1,MAXSTP
		call derivs(x,y,dydx)
		yscal(:)=abs(y(:))+abs(h*dydx(:))+TINY
		if (save_steps .and. (abs(x-xsav) > abs(dxsav))) &

			call save_a_step
		if ((x+h-x2)*(x+h-x1) > 0.0) h=x2-x
		call rkqs(y,dydx,x,h,eps,yscal,hdid,hnext,derivs)
		if (hdid == h) then
			nok=nok+1
		else
			nbad=nbad+1
		end if
		if ((x-x2)*(x2-x1) >= 0.0) then
			ystart(:)=y(:)
			if (save_steps) call save_a_step
			RETURN
		end if
		if (abs(hnext) < hmin)&
!            hnext=hnext
!			call nrerror('stepsize smaller than minimum in odeint')
		h=hnext
	end do
	call nrerror('too many steps in odeint')
	CONTAINS
!BL
	SUBROUTINE save_a_step
	kount=kount+1
	if (kount > size(xp)) then

		xp=>reallocate(xp,2*size(xp))
		yp=>reallocate(yp,size(yp,1),size(xp))
	end if
	xp(kount)=x
	yp(:,kount)=y(:)
	xsav=x
	END SUBROUTINE save_a_step
	END SUBROUTINE odeint




!**************************************
	SUBROUTINE polin2(x1a,x2a,ya,x1,x2,y,dy)
	USE nrtype; USE nrutil, ONLY : assert_eq
	USE nr, ONLY : polint
	IMPLICIT NONE
	REAL(SP), DIMENSION(:), INTENT(IN) :: x1a,x2a
	REAL(SP), DIMENSION(:,:), INTENT(IN) :: ya
	REAL(SP), INTENT(IN) :: x1,x2
	REAL(SP), INTENT(OUT) :: y,dy
	INTEGER(I4B) :: j,m,ndum
	REAL(SP), DIMENSION(size(x1a)) :: ymtmp
	REAL(SP), DIMENSION(size(x2a)) :: yntmp
	m=assert_eq(size(x1a),size(ya,1),'polin2: m')
	ndum=assert_eq(size(x2a),size(ya,2),'polin2: ndum')
	do j=1,m
		yntmp=ya(j,:)
		call polint(x2a,yntmp,x2,ymtmp(j),dy)
	end do
	call polint(x1a,ymtmp,x1,y,dy)
	END SUBROUTINE polin2



!**************************************
	SUBROUTINE polint(xa,ya,x,y,dy)
	USE nrtype; USE nrutil, ONLY : assert_eq,iminloc,nrerror
	IMPLICIT NONE
	REAL(SP), DIMENSION(:), INTENT(IN) :: xa,ya
	REAL(SP), INTENT(IN) :: x
	REAL(SP), INTENT(OUT) :: y,dy
	INTEGER(I4B) :: m,n,ns
	REAL(SP), DIMENSION(size(xa)) :: c,d,den,ho
	n=assert_eq(size(xa),size(ya),'polint')
	c=ya
	d=ya
	ho=xa-x
	ns=iminloc(abs(x-xa))
	y=ya(ns)
	ns=ns-1
	do m=1,n-1
		den(1:n-m)=ho(1:n-m)-ho(1+m:n)
		if (any(den(1:n-m) == 0.0)) &
			call nrerror('polint: calculation failure')
		den(1:n-m)=(c(2:n-m+1)-d(1:n-m))/den(1:n-m)
		d(1:n-m)=ho(1+m:n)*den(1:n-m)
		c(1:n-m)=ho(1:n-m)*den(1:n-m)
		if (2*ns < n-m) then

			dy=c(ns+1)
		else
			dy=d(ns)
			ns=ns-1
		end if
		y=y+dy
	end do
	END SUBROUTINE polint


!**************************************
	FUNCTION qromb(func,a,b)
	USE nrtype; USE nrutil, ONLY : nrerror
	USE nr, ONLY : polint,trapzd
	IMPLICIT NONE
	REAL(SP), INTENT(IN) :: a,b
	REAL(SP) :: qromb
	INTERFACE
		FUNCTION func(x)
		USE nrtype
		REAL(SP), DIMENSION(:), INTENT(IN) :: x
		REAL(SP), DIMENSION(size(x)) :: func
		END FUNCTION func
	END INTERFACE
	INTEGER(I4B), PARAMETER :: JMAX=20,JMAXP=JMAX+1,K=5,KM=K-1
	REAL(SP), PARAMETER :: EPS=1.0e-6_sp
	REAL(SP), DIMENSION(JMAXP) :: h,s
	REAL(SP) :: dqromb
	INTEGER(I4B) :: j
	h(1)=1.0
	do j=1,JMAX
		call trapzd(func,a,b,s(j),j)
		if (j >= K) then
			call polint(h(j-KM:j),s(j-KM:j),0.0_sp,qromb,dqromb)
			if (abs(dqromb) <= EPS*abs(qromb)) RETURN
		end if
		s(j+1)=s(j)
		h(j+1)=0.25_sp*h(j)
	end do
	call nrerror('qromb: too many steps')
	END FUNCTION qromb


!**************************************
	SUBROUTINE rkck(y,dydx,x,h,yout,yerr,derivs)
	USE nrtype; USE nrutil, ONLY : assert_eq
	IMPLICIT NONE
	REAL(SP), DIMENSION(:), INTENT(IN) :: y,dydx
	REAL(SP), INTENT(IN) :: x,h
	REAL(SP), DIMENSION(:), INTENT(OUT) :: yout,yerr
	INTERFACE
		SUBROUTINE derivs(x,y,dydx)
		USE nrtype
		IMPLICIT NONE
		REAL(SP), INTENT(IN) :: x
		REAL(SP), DIMENSION(:), INTENT(IN) :: y
		REAL(SP), DIMENSION(:), INTENT(OUT) :: dydx
		END SUBROUTINE derivs
	END INTERFACE
	INTEGER(I4B) :: ndum
	REAL(SP), DIMENSION(size(y)) :: ak2,ak3,ak4,ak5,ak6,ytemp
	REAL(SP), PARAMETER :: A2=0.2_sp,A3=0.3_sp,A4=0.6_sp,A5=1.0_sp,&
		A6=0.875_sp,B21=0.2_sp,B31=3.0_sp/40.0_sp,B32=9.0_sp/40.0_sp,&
		B41=0.3_sp,B42=-0.9_sp,B43=1.2_sp,B51=-11.0_sp/54.0_sp,&
		B52=2.5_sp,B53=-70.0_sp/27.0_sp,B54=35.0_sp/27.0_sp,&
		B61=1631.0_sp/55296.0_sp,B62=175.0_sp/512.0_sp,&
		B63=575.0_sp/13824.0_sp,B64=44275.0_sp/110592.0_sp,&

		B65=253.0_sp/4096.0_sp,C1=37.0_sp/378.0_sp,&
		C3=250.0_sp/621.0_sp,C4=125.0_sp/594.0_sp,&
		C6=512.0_sp/1771.0_sp,DC1=C1-2825.0_sp/27648.0_sp,&
		DC3=C3-18575.0_sp/48384.0_sp,DC4=C4-13525.0_sp/55296.0_sp,&
		DC5=-277.0_sp/14336.0_sp,DC6=C6-0.25_sp
	ndum=assert_eq(size(y),size(dydx),size(yout),size(yerr),'rkck')
	ytemp=y+B21*h*dydx
	call derivs(x+A2*h,ytemp,ak2)
	ytemp=y+h*(B31*dydx+B32*ak2)
	call derivs(x+A3*h,ytemp,ak3)
	ytemp=y+h*(B41*dydx+B42*ak2+B43*ak3)
	call derivs(x+A4*h,ytemp,ak4)

	ytemp=y+h*(B51*dydx+B52*ak2+B53*ak3+B54*ak4)
	call derivs(x+A5*h,ytemp,ak5)
	ytemp=y+h*(B61*dydx+B62*ak2+B63*ak3+B64*ak4+B65*ak5)
	call derivs(x+A6*h,ytemp,ak6)
	yout=y+h*(C1*dydx+C3*ak3+C4*ak4+C6*ak6)
	yerr=h*(DC1*dydx+DC3*ak3+DC4*ak4+DC5*ak5+DC6*ak6)
	END SUBROUTINE rkck



!**************************************
	SUBROUTINE rkqs(y,dydx,x,htry,eps,yscal,hdid,hnext,derivs)
	USE nrtype; USE nrutil, ONLY : assert_eq,nrerror
	USE nr, ONLY : rkck
	IMPLICIT NONE
	REAL(SP), DIMENSION(:), INTENT(INOUT) :: y
	REAL(SP), DIMENSION(:), INTENT(IN) :: dydx,yscal
	REAL(SP), INTENT(INOUT) :: x
	REAL(SP), INTENT(IN) :: htry,eps
	REAL(SP), INTENT(OUT) :: hdid,hnext
	INTERFACE
		SUBROUTINE derivs(x,y,dydx)
		USE nrtype
		IMPLICIT NONE
		REAL(SP), INTENT(IN) :: x
		REAL(SP), DIMENSION(:), INTENT(IN) :: y
		REAL(SP), DIMENSION(:), INTENT(OUT) :: dydx
		END SUBROUTINE derivs
	END INTERFACE
	INTEGER(I4B) :: ndum
	REAL(SP) :: errmax,h,htemp,xnew
	REAL(SP), DIMENSION(size(y)) :: yerr,ytemp
	REAL(SP), PARAMETER :: SAFETY=0.9_sp,PGROW=-0.2_sp,PSHRNK=-0.25_sp,&
		ERRCON=1.89e-4
	ndum=assert_eq(size(y),size(dydx),size(yscal),'rkqs')
	h=htry
	do
		call rkck(y,dydx,x,h,ytemp,yerr,derivs)
		errmax=maxval(abs(yerr(:)/yscal(:)))/eps
		if (errmax <= 1.0) exit
		htemp=SAFETY*h*(errmax**PSHRNK)
		h=sign(max(abs(htemp),0.1_sp*abs(h)),h)

		xnew=x+h
		if (xnew == x) call nrerror('stepsize underflow in rkqs')
	end do
	if (errmax > ERRCON) then
		hnext=SAFETY*h*(errmax**PGROW)
	else
		hnext=5.0_sp*h
	end if
	hdid=h
	x=x+h
	y(:)=ytemp(:)
	END SUBROUTINE rkqs



!**************************************
	SUBROUTINE splie2(x1a,x2a,ya,y2a)
	USE nrtype; USE nrutil, ONLY : assert_eq
	USE nr, ONLY : spline
	IMPLICIT NONE
	REAL(SP), DIMENSION(:), INTENT(IN) :: x1a,x2a
	REAL(SP), DIMENSION(:,:), INTENT(IN) :: ya
	REAL(SP), DIMENSION(:,:), INTENT(OUT) :: y2a
	INTEGER(I4B) :: j,m,ndum
	m=assert_eq(size(x1a),size(ya,1),size(y2a,1),'splie2: m')
	ndum=assert_eq(size(x2a),size(ya,2),size(y2a,2),'splie2: ndum')
	do j=1,m
		call spline(x2a,ya(j,:),1.0e30_sp,1.0e30_sp,y2a(j,:))
	end do
	END SUBROUTINE splie2



!**************************************
	FUNCTION splin2(x1a,x2a,ya,y2a,x1,x2)
	USE nrtype; USE nrutil, ONLY : assert_eq
	USE nr, ONLY : spline,splint
	IMPLICIT NONE
	REAL(SP), DIMENSION(:), INTENT(IN) :: x1a,x2a
	REAL(SP), DIMENSION(:,:), INTENT(IN) :: ya,y2a
	REAL(SP), INTENT(IN) :: x1,x2
	REAL(SP) :: splin2
	INTEGER(I4B) :: j,m,ndum
	REAL(SP), DIMENSION(size(x1a)) :: yytmp,y2tmp2
	m=assert_eq(size(x1a),size(ya,1),size(y2a,1),'splin2: m')
	ndum=assert_eq(size(x2a),size(ya,2),size(y2a,2),'splin2: ndum')
	do j=1,m
		yytmp(j)=splint(x2a,ya(j,:),y2a(j,:),x2)
	end do
	call spline(x1a,yytmp,1.0e30_sp,1.0e30_sp,y2tmp2)
	splin2=splint(x1a,yytmp,y2tmp2,x1)
	END FUNCTION splin2


!**************************************
	SUBROUTINE spline(x,y,yp1,ypn,y2)
	USE nrtype; USE nrutil, ONLY : assert_eq
	USE nr, ONLY : tridag
	IMPLICIT NONE
	REAL(SP), DIMENSION(:), INTENT(IN) :: x,y
	REAL(SP), INTENT(IN) :: yp1,ypn
	REAL(SP), DIMENSION(:), INTENT(OUT) :: y2
	INTEGER(I4B) :: n
	REAL(SP), DIMENSION(size(x)) :: a,b,c,r
	n=assert_eq(size(x),size(y),size(y2),'spline')
	c(1:n-1)=x(2:n)-x(1:n-1)
	r(1:n-1)=6.0_sp*((y(2:n)-y(1:n-1))/c(1:n-1))
	r(2:n-1)=r(2:n-1)-r(1:n-2)
	a(2:n-1)=c(1:n-2)
	b(2:n-1)=2.0_sp*(c(2:n-1)+a(2:n-1))
	b(1)=1.0
	b(n)=1.0
	if (yp1 > 0.99e30_sp) then
		r(1)=0.0
		c(1)=0.0
	else
		r(1)=(3.0_sp/(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-yp1)
		c(1)=0.5
	end if

	if (ypn > 0.99e30_sp) then
		r(n)=0.0
		a(n)=0.0
	else
		r(n)=(-3.0_sp/(x(n)-x(n-1)))*((y(n)-y(n-1))/(x(n)-x(n-1))-ypn)
		a(n)=0.5
	end if
	call tridag(a(2:n),b(1:n),c(1:n-1),r(1:n),y2(1:n))
	END SUBROUTINE spline



!**************************************
	FUNCTION splint(xa,ya,y2a,x)
	USE nrtype; USE nrutil, ONLY : assert_eq,nrerror
	USE nr, ONLY: locate
	IMPLICIT NONE
	REAL(SP), DIMENSION(:), INTENT(IN) :: xa,ya,y2a
	REAL(SP), INTENT(IN) :: x
	REAL(SP) :: splint
	INTEGER(I4B) :: khi,klo,n
	REAL(SP) :: a,b,h
	n=assert_eq(size(xa),size(ya),size(y2a),'splint')
	klo=max(min(locate(xa,x),n-1),1)
	khi=klo+1
	h=xa(khi)-xa(klo)
	if (h == 0.0) call nrerror('bad xa input in splint')
	a=(xa(khi)-x)/h
	b=(x-xa(klo))/h
	splint=a*ya(klo)+b*ya(khi)+((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h**2)/6.0_sp
	END FUNCTION splint



!**************************************
	SUBROUTINE trapzd(func,a,b,s,n)
	USE nrtype; USE nrutil, ONLY : arth
	IMPLICIT NONE
	REAL(SP), INTENT(IN) :: a,b
	REAL(SP), INTENT(INOUT) :: s
	INTEGER(I4B), INTENT(IN) :: n
	INTERFACE
		FUNCTION func(x)
		USE nrtype
		REAL(SP), DIMENSION(:), INTENT(IN) :: x
		REAL(SP), DIMENSION(size(x)) :: func
		END FUNCTION func
	END INTERFACE
	REAL(SP) :: del,fsum
	INTEGER(I4B) :: it
	if (n == 1) then
		s=0.5_sp*(b-a)*sum(func( (/ a,b /) ))
	else
		it=2**(n-2)
		del=(b-a)/it
		fsum=sum(func(arth(a+0.5_sp*del,del,it)))
		s=0.5_sp*(s+del*fsum)
	end if
	END SUBROUTINE trapzd




!**************************************
	SUBROUTINE tridag_ser(a,b,c,r,u)
	USE nrtype; USE nrutil, ONLY : assert_eq,nrerror
	IMPLICIT NONE
	REAL(SP), DIMENSION(:), INTENT(IN) :: a,b,c,r
	REAL(SP), DIMENSION(:), INTENT(OUT) :: u
	REAL(SP), DIMENSION(size(b)) :: gam
	INTEGER(I4B) :: n,j
	REAL(SP) :: bet
	n=assert_eq((/size(a)+1,size(b),size(c)+1,size(r),size(u)/),'tridag_ser')
	bet=b(1)
	if (bet == 0.0) call nrerror('tridag_ser: Error at code stage 1')
	u(1)=r(1)/bet
	do j=2,n
		gam(j)=c(j-1)/bet
		bet=b(j)-a(j-1)*gam(j)
		if (bet == 0.0) &
			call nrerror('tridag_ser: Error at code stage 2')
		u(j)=(r(j)-a(j-1)*u(j-1))/bet
	end do
	do j=n-1,1,-1
		u(j)=u(j)-gam(j+1)*u(j+1)
	end do
	END SUBROUTINE tridag_ser
   
	RECURSIVE SUBROUTINE tridag_par(a,b,c,r,u)
	USE nrtype; USE nrutil, ONLY : assert_eq,nrerror
	USE nr, ONLY : tridag_ser
	IMPLICIT NONE
	REAL(SP), DIMENSION(:), INTENT(IN) :: a,b,c,r
	REAL(SP), DIMENSION(:), INTENT(OUT) :: u

	INTEGER(I4B), PARAMETER :: NPAR_TRIDAG=4
	INTEGER(I4B) :: n,n2,nm,nx
	REAL(SP), DIMENSION(size(b)/2) :: y,q,piva
	REAL(SP), DIMENSION(size(b)/2-1) :: x,z
	REAL(SP), DIMENSION(size(a)/2) :: pivc
	n=assert_eq((/size(a)+1,size(b),size(c)+1,size(r),size(u)/),'tridag_par')
	if (n < NPAR_TRIDAG) then
		call tridag_ser(a,b,c,r,u)
	else
		if (maxval(abs(b(1:n))) == 0.0) &
			call nrerror('tridag_par: possible singular matrix')
		n2=size(y)
		nm=size(pivc)
		nx=size(x)
		piva = a(1:n-1:2)/b(1:n-1:2)
		pivc = c(2:n-1:2)/b(3:n:2)
		y(1:nm) = b(2:n-1:2)-piva(1:nm)*c(1:n-2:2)-pivc*a(2:n-1:2)
		q(1:nm) = r(2:n-1:2)-piva(1:nm)*r(1:n-2:2)-pivc*r(3:n:2)

		if (nm < n2) then
			y(n2) = b(n)-piva(n2)*c(n-1)
			q(n2) = r(n)-piva(n2)*r(n-1)
		end if
		x = -piva(2:n2)*a(2:n-2:2)
		z = -pivc(1:nx)*c(3:n-1:2)
		call tridag_par(x,y,z,q,u(2:n:2))
		u(1) = (r(1)-c(1)*u(2))/b(1)
		u(3:n-1:2) = (r(3:n-1:2)-a(2:n-2:2)*u(2:n-2:2) &
			-c(3:n-1:2)*u(4:n:2))/b(3:n-1:2)
		if (nm == n2) u(n)=(r(n)-a(n-1)*u(n-1))/b(n)
	end if
	END SUBROUTINE tridag_par



!**************************************
	FUNCTION zbrent(func,x1,x2,tol)
	USE nrtype; USE nrutil, ONLY : nrerror
	IMPLICIT NONE
	REAL(SP), INTENT(IN) :: x1,x2,tol
	REAL(SP) :: zbrent
	INTERFACE
		FUNCTION func(x)
		USE nrtype
		IMPLICIT NONE
		REAL(SP), INTENT(IN) :: x
		REAL(SP) :: func
		END FUNCTION func
	END INTERFACE
	INTEGER(I4B), PARAMETER :: ITMAX=100
	REAL(SP), PARAMETER :: EPS=epsilon(x1)
	INTEGER(I4B) :: iter
	REAL(SP) :: a,b,c,d,e,fa,fb,fc,p,q,r,s,tol1,xm
	a=x1
	b=x2
	fa=func(a)
	fb=func(b)
	if ((fa > 0.0 .and. fb > 0.0) .or. (fa < 0.0 .and. fb < 0.0)) &
		call nrerror('root must be bracketed for zbrent')
	c=b
	fc=fb
	do iter=1,ITMAX
		if ((fb > 0.0 .and. fc > 0.0) .or. (fb < 0.0 .and. fc < 0.0)) then
			c=a
			fc=fa
			d=b-a
			e=d
		end if
		if (abs(fc) < abs(fb)) then

			a=b
			b=c
			c=a
			fa=fb
			fb=fc
			fc=fa
		end if
		tol1=2.0_sp*EPS*abs(b)+0.5_sp*tol
		xm=0.5_sp*(c-b)
		if (abs(xm) <= tol1 .or. fb == 0.0) then
			zbrent=b
			RETURN
		end if
		if (abs(e) >= tol1 .and. abs(fa) > abs(fb)) then
			s=fb/fa
			if (a == c) then
				p=2.0_sp*xm*s
				q=1.0_sp-s
			else
				q=fa/fc
				r=fb/fc
				p=s*(2.0_sp*xm*q*(q-r)-(b-a)*(r-1.0_sp))
				q=(q-1.0_sp)*(r-1.0_sp)*(s-1.0_sp)

			end if
			if (p > 0.0) q=-q
			p=abs(p)
			if (2.0_sp*p  <  min(3.0_sp*xm*q-abs(tol1*q),abs(e*q))) then
				e=d
				d=p/q
			else
				d=xm
				e=d
			end if
		else
			d=xm
			e=d
		end if
		a=b
		fa=fb
		b=b+merge(d,sign(tol1,xm), abs(d) > tol1 )
		fb=func(b)
	end do
	call nrerror('zbrent: exceeded maximum iterations')
	zbrent=b
	END FUNCTION zbrent


!**************************************
	FUNCTION erfcc_s(x)
	USE nrtype; USE nrutil, ONLY : poly
	IMPLICIT NONE
	REAL(SP), INTENT(IN) :: x
	REAL(SP) :: erfcc_s
	REAL(SP) :: t,z
	REAL(SP), DIMENSION(10) :: coef = (/-1.26551223_sp,1.00002368_sp,&
		0.37409196_sp,0.09678418_sp,-0.18628806_sp,0.27886807_sp,&
		-1.13520398_sp,1.48851587_sp,-0.82215223_sp,0.17087277_sp/)
	z=abs(x)
    t=1.0_sp/(1.0_sp+0.5_sp*z)
    erfcc_s=t*exp(-z*z+poly(t,coef))
	if (x < 0.0) then
        erfcc_s=2.0_sp-erfcc_s
    endif
	END FUNCTION erfcc_s
  

!**************************************
	SUBROUTINE amoeba(p,y,ftol,func,iter)
	USE nrtype; USE nrutil, ONLY : assert_eq,imaxloc,iminloc,nrerror,swap
	IMPLICIT NONE
	INTEGER(I4B), INTENT(OUT) :: iter
	REAL(SP), INTENT(IN) :: ftol
	REAL(SP), DIMENSION(:), INTENT(INOUT) :: y
	REAL(SP), DIMENSION(:,:), INTENT(INOUT) :: p
	INTERFACE
		FUNCTION func(x)
		USE nrtype
		IMPLICIT NONE
		REAL(SP), DIMENSION(:), INTENT(IN) :: x
		REAL(SP) :: func
		END FUNCTION func
	END INTERFACE
	INTEGER(I4B), PARAMETER :: ITMAX=5000
	INTEGER(I4B) :: ihi,ndim
	REAL(SP), DIMENSION(size(p,2)) :: psum
	call amoeba_private
	CONTAINS
!BL
	SUBROUTINE amoeba_private
	IMPLICIT NONE
	INTEGER(I4B) :: i,ilo,inhi
	REAL(SP) :: rtol,ysave,ytry,ytmp
	ndim=assert_eq(size(p,2),size(p,1)-1,size(y)-1,'amoeba')
	iter=0
	psum(:)=sum(p(:,:),dim=1)
	do
		ilo=iminloc(y(:))
		ihi=imaxloc(y(:))
		ytmp=y(ihi)

		y(ihi)=y(ilo)
		inhi=imaxloc(y(:))
		y(ihi)=ytmp
		rtol=2.0_sp*abs(y(ihi)-y(ilo))/(abs(y(ihi))+abs(y(ilo)))
		if (rtol < ftol) then
			call swap(y(1),y(ilo))
			call swap(p(1,:),p(ilo,:))
			RETURN
		end if
		if (iter >= ITMAX) call nrerror('ITMAX exceeded in amoeba')
		ytry=amotry(-1.0_sp)
		iter=iter+1
		if (ytry <= y(ilo)) then
			ytry=amotry(2.0_sp)
			iter=iter+1
		else if (ytry >= y(inhi)) then
			ysave=y(ihi)
			ytry=amotry(0.5_sp)
			iter=iter+1

			if (ytry >= ysave) then
				p(:,:)=0.5_sp*(p(:,:)+spread(p(ilo,:),1,size(p,1)))
				do i=1,ndim+1
					if (i /= ilo) y(i)=func(p(i,:))
				end do
				iter=iter+ndim
				psum(:)=sum(p(:,:),dim=1)
			end if
		end if
	end do
	END SUBROUTINE amoeba_private
!BL
	FUNCTION amotry(fac)
	IMPLICIT NONE
	REAL(SP), INTENT(IN) :: fac
	REAL(SP) :: amotry
	REAL(SP) :: fac1,fac2,ytry
	REAL(SP), DIMENSION(size(p,2)) :: ptry
	fac1=(1.0_sp-fac)/ndim
	fac2=fac1-fac
	ptry(:)=psum(:)*fac1-p(ihi,:)*fac2
	ytry=func(ptry)
	if (ytry < y(ihi)) then

		y(ihi)=ytry
		psum(:)=psum(:)-p(ihi,:)+ptry(:)
		p(ihi,:)=ptry(:)
	end if
	amotry=ytry
	END FUNCTION amotry
	END SUBROUTINE amoeba

!----------------------------------------------------------
	FUNCTION qromo(func,a,b,choose)
	USE nrtype
    USE nrutil, ONLY : nrerror
	USE nr, ONLY : polint
	IMPLICIT NONE
	REAL(SP), INTENT(IN) :: a,b
	REAL(SP) :: qromo
	INTERFACE
		FUNCTION func(x)
		USE nrtype
		IMPLICIT NONE
		REAL(SP), DIMENSION(:), INTENT(IN) :: x
		REAL(SP), DIMENSION(size(x)) :: func
		END FUNCTION func
!BL
		SUBROUTINE choose(funk,aa,bb,s,n)
		USE nrtype
		IMPLICIT NONE
		REAL(SP), INTENT(IN) :: aa,bb
		REAL(SP), INTENT(INOUT) :: s
		INTEGER(I4B), INTENT(IN) :: n
		INTERFACE
			FUNCTION funk(x)
			USE nrtype
			IMPLICIT NONE
			REAL(SP), DIMENSION(:), INTENT(IN) :: x
			REAL(SP), DIMENSION(size(x)) :: funk
			END FUNCTION funk
		END INTERFACE
		END SUBROUTINE choose
	END INTERFACE
	INTEGER(I4B), PARAMETER :: JMAX=14,JMAXP=JMAX+1,K=5,KM=K-1
	REAL(SP), PARAMETER :: EPS=1.0e-6
	REAL(SP), DIMENSION(JMAXP) :: h,s
	REAL(SP) :: dqromo
	INTEGER(I4B) :: j
	h(1)=1.0
	do j=1,JMAX
		call choose(func,a,b,s(j),j)
		if (j >= K) then
			call polint(h(j-KM:j),s(j-KM:j),0.0_sp,qromo,dqromo)
			if (abs(dqromo) <= EPS*abs(qromo)) RETURN

		end if
		s(j+1)=s(j)
		h(j+1)=h(j)/9.0_sp
	end do
	call nrerror('qromo: too many steps')
	END FUNCTION qromo


!----------------------------------------------------------
	SUBROUTINE midinf(funk,aa,bb,s,n)
	USE nrtype
    USE nrutil, ONLY : arth,assert
	IMPLICIT NONE
	REAL(SP), INTENT(IN) :: aa,bb
	REAL(SP), INTENT(INOUT) :: s
	INTEGER(I4B), INTENT(IN) :: n
	INTERFACE
		FUNCTION funk(x)
		USE nrtype
		REAL(SP), DIMENSION(:), INTENT(IN) :: x
		REAL(SP), DIMENSION(size(x)) :: funk
		END FUNCTION funk
	END INTERFACE
	REAL(SP) :: a,b,del
	INTEGER(I4B) :: it
	REAL(SP), DIMENSION(2*3**(n-2)) :: x
	call assert(aa*bb > 0.0, 'midinf args')
	b=1.0_sp/aa
	a=1.0_sp/bb
	if (n == 1) then
		s=(b-a)*sum(func( (/0.5_sp*(a+b)/) ))
	else
		it=3**(n-2)
		del=(b-a)/(3.0_sp*it)
		x(1:2*it-1:2)=arth(a+0.5_sp*del,3.0_sp*del,it)
		x(2:2*it:2)=x(1:2*it-1:2)+2.0_sp*del
		s=s/3.0_sp+del*sum(func(x))
	end if
	CONTAINS
!BL
		FUNCTION func(x)
		REAL(SP), DIMENSION(:), INTENT(IN) :: x
		REAL(SP), DIMENSION(size(x)) :: func
		func=funk(1.0_sp/x)/x**2
		END FUNCTION func
	END SUBROUTINE midinf

!----------------------------------------------------------
	SUBROUTINE midpnt(func,a,b,s,n)
	USE nrtype
    USE nrutil, ONLY : arth
	IMPLICIT NONE
	REAL(SP), INTENT(IN) :: a,b
	REAL(SP), INTENT(INOUT) :: s
	INTEGER(I4B), INTENT(IN) :: n
	INTERFACE
		FUNCTION func(x)
		USE nrtype
		REAL(SP), DIMENSION(:), INTENT(IN) :: x
		REAL(SP), DIMENSION(size(x)) :: func
		END FUNCTION func
	END INTERFACE
	REAL(SP) :: del
	INTEGER(I4B) :: it
	REAL(SP), DIMENSION(2*3**(n-2)) :: x
	if (n == 1) then
		s=(b-a)*sum(func( (/0.5_sp*(a+b)/) ))
	else
		it=3**(n-2)
		del=(b-a)/(3.0_sp*it)
		x(1:2*it-1:2)=arth(a+0.5_sp*del,3.0_sp*del,it)
		x(2:2*it:2)=x(1:2*it-1:2)+2.0_sp*del
		s=s/3.0_sp+del*sum(func(x))
	end if
	END SUBROUTINE midpnt



MODULE rkdumb_path
	USE nrtype
	REAL(SP), DIMENSION(:), ALLOCATABLE:: xx
	REAL(SP), DIMENSION(:,:), ALLOCATABLE :: y
END MODULE rkdumb_path
   
	SUBROUTINE rkdumb(vstart,x1,x2,nstep,derivs)
	USE nrtype; USE nrutil, ONLY : nrerror
	USE nr, ONLY : rk4
	USE rkdumb_path
	IMPLICIT NONE
	REAL(SP), DIMENSION(:), INTENT(IN) :: vstart
	REAL(SP), INTENT(IN) :: x1,x2
	INTEGER(I4B), INTENT(IN) :: nstep
	INTERFACE
		SUBROUTINE derivs(x,y,dydx)
		USE nrtype
		IMPLICIT NONE
		REAL(SP), INTENT(IN) :: x
		REAL(SP), DIMENSION(:), INTENT(IN) :: y
		REAL(SP), DIMENSION(:), INTENT(OUT) :: dydx
		END SUBROUTINE derivs
	END INTERFACE
	INTEGER(I4B) :: k
	REAL(SP) :: h,x
	REAL(SP), DIMENSION(size(vstart)) :: dv,v
	v(:)=vstart(:)
	if (allocated(xx)) deallocate(xx)
	if (allocated(y)) deallocate(y)
	allocate(xx(nstep+1))
	allocate(y(size(vstart),nstep+1))
	y(:,1)=v(:)
	xx(1)=x1
	x=x1

	h=(x2-x1)/nstep
	do k=1,nstep
		call derivs(x,v,dv)
		call rk4(v,dv,x,h,v,derivs)
		if (x+h == x) call nrerror('stepsize not significant in rkdumb')
		x=x+h
		xx(k+1)=x
		y(:,k+1)=v(:)
	end do
	END SUBROUTINE rkdumb





SUBROUTINE rk4(y,dydx,x,h,yout,derivs)
	USE nrtype; USE nrutil, ONLY : assert_eq
	IMPLICIT NONE
	REAL(SP), DIMENSION(:), INTENT(IN) :: y,dydx
	REAL(SP), INTENT(IN) :: x,h
	REAL(SP), DIMENSION(:), INTENT(OUT) :: yout
	INTERFACE
		SUBROUTINE derivs(x,y,dydx)
		USE nrtype
		IMPLICIT NONE
		REAL(SP), INTENT(IN) :: x
		REAL(SP), DIMENSION(:), INTENT(IN) :: y
		REAL(SP), DIMENSION(:), INTENT(OUT) :: dydx
		END SUBROUTINE derivs
	END INTERFACE
	INTEGER(I4B) :: ndum
	REAL(SP) :: h6,hh,xh
	REAL(SP), DIMENSION(size(y)) :: dym,dyt,yt
	ndum=assert_eq(size(y),size(dydx),size(yout),'rk4')
	hh=h*0.5_sp
	h6=h/6.0_sp
	xh=x+hh
	yt=y+hh*dydx
	call derivs(xh,yt,dyt)
	yt=y+hh*dyt
	call derivs(xh,yt,dym)
	yt=y+h*dym
	dym=dyt+dym
	call derivs(x+h,yt,dyt)
	yout=y+h6*(dydx+dyt+2.0_sp*dym)
	END SUBROUTINE rk4


              